//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : OLED Display control functions
// Includes  : i2cmaster.h, display.h
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Sprache   : C
// Datum     : 24.04.2014 22:19:02
// Version   : 1.02
// Autor     : Andreas Nestler
// Changes   :
//		v1   : initial version; Display extracted control functions
//      v1.01: some small updates to avoid globals and extern
//      v1.02: new int to ascii routine to speed up 
//             and to avoid old data fragments
//      v1.03: header file update after header files cleanup
//----------------------------------------------------------------------

#include "inc/globals.h"  // before delay.h, da F_CPU benoetigt wird
#include <util/delay.h>   // for delay functions, requires F_CPU definition
#include "inc/i2cmaster.h"    // for i2c functions
#include "inc/display.h"
//---------------FONT + GRAPHIC-----------------------------//
#include "inc/display_data.h"

#include <avr/pgmspace.h>

// global definition for strings means here less memory
/* char oledString1[]="RSSI:    "; 
char oledString2[]="SNR:     "; 
char oledString3[]="STBLEND: "; */

// char charBuf[50];

void print_a_char(unsigned char ascii)
{
	unsigned char i=0;
	for(i=0;i<8;i++)
	{
		SendChar(pgm_read_byte(myFont[ascii-0x20]+i));
	}
}

void setup_OLED(void)
{
	
	// Wire.begin();
	init_OLED();
	_delay_ms(10);
	ssd1306_command(SSD1306_DISPLAYOFF);			 // set display off to hide the display build-up
	clear_display();
	_delay_ms(50);
	
	ssd1306_command(SSD1306_MEMORYMODE);	//--0x20 Set Memory Addressing Mode
	ssd1306_command(0x02);	//00,Horizontal Addressing Mode;01,Vertical Addressing Mode;10,Page Addressing Mode (RESET);11,Invalid
	ssd1306_command(SSD1306_NORMALDISPLAY); // 0xa6
	
/*	//setXY(2,1);
	sendStrXY(2,1,oledString1);
	
	//setXY(4,1);
	sendStrXY(4,1,oledString2);
	
	sendStrXY(6,1,oledString3);
	
	ssd1306_command(SSD1306_DISPLAYON);			 // set display on
	*/
}




void ssd1306_command(char com)
{
	i2c_start(SSD1306_I2C_ADDRESS+I2C_WRITE);    // begin transmitting
	i2c_write(0x00);                          // command mode - Co = 0, Dc = 0
	i2c_write(com);
	i2c_stop();                    // stop transmitting
	_delay_ms(1);
}

void clear_display(void)
{
	unsigned char i,k;
	for(k=0;k<8;k++)
	{
		setXY(k,0);
		{
			for(i=0;i<128;i++)     //clear all COL
			{
				SendChar(0);         //clear all COL
				//delay(10);
			}
		}
	}
}

//==========================================================//
void SendChar(char data)
{
	i2c_start(SSD1306_I2C_ADDRESS+I2C_WRITE); // begin transmitting
	i2c_write(0x40);//data mode
	i2c_write(data);
	i2c_stop();    // stop transmitting
	// _delay_ms(1);
}

//==========================================================//
void setXY(unsigned char row,unsigned char col)
{
	ssd1306_command(0xB0+row);                //set page address
	ssd1306_command(0x00+(8*col&0x0F));       //set low col address
	ssd1306_command(0x10+((8*col>>4)&0x0F));  //set high col address
}


//==========================================================//
void sendStrXY(unsigned char row,unsigned char col,char *string)
{
	unsigned char i=0;
	
	setXY(row,col);
	while(*string)
	{
		for(i=0;i<8;i++)
		{
			   SendChar(pgm_read_byte(myFont[*string-0x20]+i)); // pass the pointer address to pgm_read_byte function
			
			 // SendChar(*string);
			_delay_ms(1);
		}
		string++;  // pointer to next char, orig *string++;
	}
}

//==========================================================//
void init_OLED(void)
{
    // ssd1306_command(0xae);		//display off
    // ssd1306_command(0xa6);            //Set Normal Display (default)
    // Adafruit Init sequence for 128x64 OLED module
    ssd1306_command(0xAE);             //DISPLAYOFF
    ssd1306_command(0xD5);            //SETDISPLAYCLOCKDIV
    ssd1306_command(0x80);            // the suggested ratio 0x80
    ssd1306_command(0xA8);            //SSD1306_SETMULTIPLEX
    ssd1306_command(0x3F);
    ssd1306_command(0xD3);            //SETDISPLAYOFFSET
    ssd1306_command(0x0);             //no offset
    ssd1306_command(0x40 | 0x0);      //SETSTARTLINE
    ssd1306_command(0x8D);            //CHARGEPUMP
    ssd1306_command(0x14);
    ssd1306_command(0x20);             //MEMORYMODE
    ssd1306_command(0x00);             //0x0 act like ks0108
    
    //ssd1306_command(0xA0 | 0x1);      //SEGREMAP   //Rotate screen 180 deg
    ssd1306_command(0xA0);
    
    //ssd1306_command(0xC8);            //COMSCANDEC  Rotate screen 180 Deg
    ssd1306_command(0xC0);
    
    ssd1306_command(0xDA);            //0xDA set com pins hardware configuration
    ssd1306_command(0x12);           //COMSCANDEC
    ssd1306_command(0x81);           //SETCONTRAST
    ssd1306_command(0xCF);           //
    ssd1306_command(0xd9);          //SETPRECHARGE
    ssd1306_command(0xF1);
    ssd1306_command(0xDB);        //SETVCOMDETECT
    ssd1306_command(0x40);
    ssd1306_command(0xA4);        //DISPLAYALLON_RESUME
    ssd1306_command(0xA6);        //NORMALDISPLAY
    
    clear_display();
    ssd1306_command(0x2e);            // stop scroll
    //----------------------------REVERSE comments----------------------------//
    //  ssd1306_command(0xa0);		//seg re-map 0->127(default)
    //  ssd1306_command(0xa1);		//seg re-map 127->0
    //  ssd1306_command(0xc8);
    //  delay(1000);
    //----------------------------REVERSE comments----------------------------//
    // ssd1306_command(0xa7);  //Set Inverse Display
    // ssd1306_command(0xae);		//display off
	
	#if DISPLAY_START_LOGO == 1  // setup for logo only if DISPLAY_START_LOGO == 1
    ssd1306_command(0x20);            //Set Memory Addressing Mode
    ssd1306_command(0x00);            //Set Memory Addressing Mode ab Horizontal addressing mode
    //  ssd1306_command(0x02);         // Set Memory Addressing Mode ab Page addressing mode(RESET)
    
    setXY(0,0);
    
    for(int i=0;i<128*8;i++)     // show 128* 64 Logo
    {
	    SendChar(pgm_read_byte(logo+i));
    }
	#endif // DISPLAY_START_LOGO
    ssd1306_command(0xaf);		//display on
    
	#if DISPLAY_START_LOGO == 1  // delay for logo only if DISPLAY_START_LOGO == 1
    // sendStrXY("by KaR]V[aN",7,5);
    _delay_ms(2000);
	#endif // DISPLAY_START_LOGO

}

