//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : OLED Display control functions
// Includes  : i2cmaster.h, display.h
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Sprache   : C
// Datum     : 02.04.2014 21:57:02
// Version   : 1.00
// Autor     : Andreas Nestler
// Changes   :
//		v1   : initial version; Display extracted control functions
//----------------------------------------------------------------------

#include "inc/display.h"

unsigned char fill_OLED=0x55;
//extern
char oledString1[]="RSSI: ";
//extern
char oledString2[]="STBLEND: ";

extern unsigned char myFont[][8]; // defined in data.c
extern unsigned char logo[]; // defined in data.c
// extern unsigned char ip[];

char charBuf[50];

void print_a_char(unsigned char ascii)
{
	unsigned char i=0;
	for(i=0;i<8;i++)
	{
		SendChar(myFont[ascii-0x20][i]);
	}
}

void setup_OLED(char vccstate)
{
	
	// Wire.begin();
	init_OLED(vccstate);
	_delay_ms(10);
	clear_display();
	_delay_ms(50);
	
	
	//ssd1306_command(0x20);            //Set Memory Addressing Mode
	//ssd1306_command(0x02);            //Set Memory Addressing Mode ab Page addressing mode(RESET)
	// ssd1306_command(0xA6);            //Set Normal Display (default)
	//ssd1306_command(0xA4);			 // set display on
	
	setXY(2,1);
	sendStr(oledString1);
	
	setXY(4,1);
	sendStr(oledString2);
	_delay_ms(1000);
	 ssd1306_command(SSD1306_INVERTDISPLAY);  //--0xa7 Set Inverse Display 
	_delay_ms(1000);
	
	clear_display();
	_delay_ms(50);

	ssd1306_command(0xa6);  //Set Normal Display

	ssd1306_command(0xae);		//display off
	ssd1306_command(0x20);            //Set Memory Addressing Mode
	ssd1306_command(0x00);            //Set Memory Addressing Mode ab Horizontal addressing mode


	//==========================================================//
	for(int i=0;i<128*8;i++)     // show 128* 64 picture
	{
		SendChar(logo[i]);
	}
	
	 ssd1306_command(0xaf);
	 _delay_ms(2000);
	 ssd1306_command(0xa7);    //Set Inverse Display
	 _delay_ms(2000);
	
}




void ssd1306_command(char com)
{
	i2c_start(SSD1306_I2C_ADDRESS+I2C_WRITE);    // begin transmitting
	i2c_write(0x00);                          // command mode
	i2c_write(com);
	i2c_stop();                    // stop transmitting
	_delay_ms(1);
}

void clear_display(void)
{
	unsigned char i,k;
	for(k=0;k<8;k++)
	{
		setXY(k,0);
		{
			for(i=0;i<128;i++)     //clear all COL
			{
				SendChar(0);         //clear all COL
				//delay(10);
			}
		}
	}
}

//==========================================================//
void SendChar(char data)
{
	i2c_start(SSD1306_I2C_ADDRESS+I2C_WRITE); // begin transmitting
	i2c_write(0x40);//data mode
	i2c_write(data);
	i2c_stop();    // stop transmitting
}

//==========================================================//
void setXY(unsigned char row,unsigned char col)
{
	ssd1306_command(0xB0+row);                //set page address
	ssd1306_command(0x00+(8*col&0x0F));       //set low col address
	ssd1306_command(0x10+((8*col>>4)&0x0F));  //set high col address
}


//==========================================================//
void sendStr(char *string)
{
	unsigned char i=0;
//	char c=*string;
	//setXY(0,0);
	while(*string)
	{
		for(i=0;i<8;i++)
		{
			   SendChar(myFont[*string-0x20][i]);
			
			 // SendChar(*string);
			_delay_ms(10);
		}
		string++;  // pointer to next char
//		c=*string; //
	}
}

//==========================================================//
void init_OLED(char vccstate)
{
	
/*	ssd1306_command(0xAE);        //display off
	_delay_ms(50);
	//----------------------------REVERSE comments----------------------------//
	//  ssd1306_command(0xa0);        //seg re-map 0->127(default)
	//  ssd1306_command(0xa1);        //seg re-map 127->0
	//  ssd1306_command(0xc8);
	//  delay(1000);
	//----------------------------REVERSE comments----------------------------//
	
	ssd1306_command(0xA4);        // Disable Entire	Display On
	ssd1306_command(0xA6);        // Set normal display
	ssd1306_command(0x8D);		  // Charge pump setting
	ssd1306_command(0x14);		  // Enable Charge pump
	
	ssd1306_command(0xAF);        //display on  */
	
	ssd1306_command(SSD1306_DISPLAYOFF);   //--0xae display off
    ssd1306_command(SSD1306_MEMORYMODE);	//--0x20 Set Memory Addressing Mode	
    ssd1306_command(0x10);	//00,Horizontal Addressing Mode;01,Vertical Addressing Mode;10,Page Addressing Mode (RESET);11,Invalid
    ssd1306_command(0xb0);	//Set Page Start Address for Page Addressing Mode,0-7
    ssd1306_command(SSD1306_COMSCANDEC);	//--0xc8 Set COM Output Scan Direction
    ssd1306_command(0x00);//---set low column address
    ssd1306_command(0x10);//---set high column address
    ssd1306_command(0x40);//--set start line address
    ssd1306_command(SSD1306_SETCONTRAST);//--0x81 set contrast control register
    if (vccstate == SSD1306_EXTERNALVCC)
      { ssd1306_command(0x9F); }
    else
      { ssd1306_command(0xCF); }
    // ssd1306_command(0x7f); //--contrast default
    ssd1306_command(SSD1306_SEGREMAP | 0x1);//--set segment re-map 0 to 127
    ssd1306_command(SSD1306_NORMALDISPLAY);//--0xa6 set normal display
    ssd1306_command(SSD1306_SETMULTIPLEX);//--0xa8 set multiplex ratio(1 to 64)
    ssd1306_command(0x3F);//
    ssd1306_command(SSD1306_DISPLAYALLON_RESUME);//0xa4,Output follows RAM content;0xa5,Output ignores RAM content
    ssd1306_command(SSD1306_SETDISPLAYOFFSET);//-set display offset
    ssd1306_command(0x00);//-not offset
    ssd1306_command(SSD1306_SETDISPLAYCLOCKDIV);//--0xd5 set display clock divide ratio/oscillator frequency
    ssd1306_command(0x80);//--set divide ratio
    ssd1306_command(SSD1306_SETPRECHARGE);//--set pre-charge period
    if (vccstate == SSD1306_EXTERNALVCC)
      { ssd1306_command(0x22); }
    else
      { ssd1306_command(0xF1); }
    // ssd1306_command(0x22); //

    ssd1306_command(SSD1306_SETCOMPINS);//--0xda set com pins hardware configuration
    ssd1306_command(0x12);
    ssd1306_command(SSD1306_SETVCOMDETECT);//--0xdb set vcomh
    ssd1306_command(0x20);//0x20,0.77xVcc
    ssd1306_command(SSD1306_CHARGEPUMP);//--set DC-DC enable
    if (vccstate == SSD1306_EXTERNALVCC)
      { ssd1306_command(0x10); } //set Charge Pump disable
    else
      { ssd1306_command(0x14); } //set Charge Pump enable
    // ssd1306_command(0x14);//set Charge Pump disable -> enable
    ssd1306_command(SSD1306_DISPLAYON);//--turn on oled panel 
	_delay_ms(50); 
	
}
